use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize, Clone, Debug)]
pub struct MessageId(Uuid);

impl MessageId {
    pub fn new() -> Self {
        MessageId(Uuid::new_v4())
    }
}

impl ToString for MessageId {
    fn to_string(&self) -> String {
        self.0.to_string()
    }
}

impl From<MessageId> for String {
    fn from(value: MessageId) -> Self {
        value.to_string()
    }
}

impl TryFrom<&str> for MessageId {
    type Error = serde_json::Error;
    fn try_from(value: &str) -> Result<Self, Self::Error> {
        serde_json::from_str(value)
    }
}
