use axum::{
    middleware::from_fn,
    routing::{get, put},
    Extension, Router,
};
use std::sync::Arc;

use crate::state::ApiState;

pub mod app;
pub mod element;

pub fn get_router(state: ApiState) -> Router {
    Router::new()
        // authenticated routes
        .route("/element", put(element::create))
        .route(
            "/element/:id",
            get(element::get).post(element::set).delete(element::remove),
        )
        .layer(from_fn(app::auth))
        // public / unauthenticated routes
        .route("/app/register", put(app::register))
        .layer(Extension(Arc::new(state)))
}
