use tracing::debug;

use ubisync_lib::types::PeerId;

use ubisync_lib::messages::{Message, MessageContent};

use crate::state::CommState;

pub fn handle(state: &CommState, peer: &PeerId, message: Message) {
    debug!("Handling message now: {:?}", message);
    match message.content() {
        MessageContent::Hello { peer_name } => {
            state.set_peer(peer, peer_name).expect("State failed");
        }
        MessageContent::CreateElement { id, content } => {
            state
                .add_received_element(id, content, message.id())
                .expect("State failed");
        }
        MessageContent::SetElement { id, content } => {
            state
                .update_element_content(id, content, message.id())
                .expect("State failed");
        }
        MessageContent::RemoveElement { id } => {
            state.remove_element(id).expect("State failed");
        }
    }
}
