use std::collections::BTreeMap;

use anyhow::{bail, Error};
use chrono::{DateTime, Utc};
use cozo::{DataValue, DbInstance, Num};

use crate::{api::v0::app::AppId, run_query};

pub fn add(
    db: &DbInstance,
    id: &AppId,
    last_access: &DateTime<Utc>,
    name: &str,
    description: &str,
) -> anyhow::Result<()> {
    let params = vec![
        (
            "id",
            DataValue::Str(serde_json::to_string(&id).unwrap().into()),
        ),
        (
            "last_access",
            DataValue::Num(Num::Int(last_access.timestamp())),
        ),
        ("name", DataValue::Str(name.into())),
        ("description", DataValue::Str(description.into())),
    ];

    match run_query!(
        &db,
        ":insert apps {id => last_access, name, description}",
        params,
        cozo::ScriptMutability::Mutable
    ) {
        Ok(_) => Ok(()),
        Err(report) => bail!(report),
    }
}

pub fn exists(db: &DbInstance, id: &AppId) -> anyhow::Result<bool> {
    let mut params = BTreeMap::new();
    params.insert(
        "id".to_string(),
        DataValue::Str(serde_json::to_string(&id)?.into()),
    );

    let result = db.run_script(
        "?[name] := *apps[$id, last_access, name, description]",
        params,
        cozo::ScriptMutability::Immutable,
    );

    if let Ok(rows) = result {
        return Ok(rows.rows.len() == 1);
    }

    Err(Error::msg("Could not check whether app is registered"))
}
